@@ -51,7 +51,6 @@ pub fn exit_boot_services() {
 /// Only valid for as long as the UEFI boot services are available.
 pub struct Allocator;
 
-#[allow(clippy::cast_ptr_alignment)]
 unsafe impl GlobalAlloc for Allocator {
     unsafe fn alloc(&self, layout: Layout) -> *mut u8 {
         let mem_ty = MemoryType::LOADER_DATA;
@@ -73,7 +72,7 @@ unsafe impl GlobalAlloc for Allocator {
             }
             let return_ptr = ptr.add(offset);
             // store allocated pointer before the struct
-            (return_ptr as *mut *mut u8).sub(1).write(ptr);
+            (return_ptr.cast::<*mut u8>()).sub(1).write(ptr);
             return_ptr
         } else {
             boot_services()
