@@ -25,11 +25,13 @@
 
 #![feature(try_trait_v2)]
 #![feature(abi_efiapi)]
+#![feature(maybe_uninit_slice)]
 #![feature(negative_impls)]
 #![feature(ptr_metadata)]
+#![cfg_attr(feature = "exts", feature(vec_into_raw_parts))]
 #![no_std]
 // Enable some additional warnings and lints.
-#![warn(missing_docs, unused)]
+#![warn(clippy::ptr_as_ptr, missing_docs, unused)]
 #![deny(clippy::all)]
 
 // `uefi-exts` requires access to memory allocation APIs.
