@@ -76,7 +76,6 @@ pub struct GraphicsOutput<'boot> {
     ) -> Status,
     set_mode: extern "efiapi" fn(&mut GraphicsOutput, mode: u32) -> Status,
     // Clippy correctly complains that this is too complicated, but we can't change the spec.
-    #[allow(clippy::type_complexity)]
     blt: unsafe extern "efiapi" fn(
         this: &mut GraphicsOutput,
         buffer: *mut BltPixel,
@@ -631,7 +630,8 @@ impl<'gop> FrameBuffer<'gop> {
             index.saturating_add(mem::size_of::<T>()) <= self.size,
             "Frame buffer accessed out of bounds"
         );
-        (self.base.add(index) as *mut T).write_volatile(value)
+        let ptr = self.base.add(index).cast::<T>();
+        ptr.write_volatile(value)
     }
 
     /// Read a value from the frame buffer, starting at the i-th byte
