@@ -1,8 +1,11 @@
 //! `LoadedImage` protocol.
 
 use crate::{
-    data_types::FromSliceWithNulError, proto::Protocol, table::boot::MemoryType, unsafe_guid,
-    CStr16, Handle, Status,
+    data_types::FromSliceWithNulError,
+    proto::device_path::{DevicePath, FfiDevicePath},
+    proto::Protocol,
+    table::boot::MemoryType,
+    unsafe_guid, CStr16, Handle, Status,
 };
 use core::{ffi::c_void, mem, slice};
 
@@ -17,7 +20,7 @@ pub struct LoadedImage {
 
     // Source location of the image
     device_handle: Handle,
-    _file_path: *const c_void, // TODO: not supported yet
+    file_path: *const FfiDevicePath,
     _reserved: *const c_void,
 
     // Image load options
@@ -53,6 +56,18 @@ impl LoadedImage {
         self.device_handle
     }
 
+    /// Get a reference to the `file_path`.
+    ///
+    /// Return `None` if the pointer to the file path portion specific to
+    /// DeviceHandle that the EFI Image was loaded from is null.
+    pub fn file_path(&self) -> Option<&DevicePath> {
+        if self.file_path.is_null() {
+            None
+        } else {
+            unsafe { Some(DevicePath::from_ffi_ptr(self.file_path)) }
+        }
+    }
+
     /// Get the load options of the image as a [`&CStr16`].
     ///
     /// Load options are typically used to pass command-line options as
@@ -73,7 +88,7 @@ impl LoadedImage {
         } else {
             let s = unsafe {
                 slice::from_raw_parts(
-                    self.load_options as *const u16,
+                    self.load_options.cast::<u16>(),
                     load_options_size / mem::size_of::<u16>(),
                 )
             };
