@@ -41,7 +41,7 @@ trait InfoInternal: Align + ptr::Pointee<Metadata = usize> {
     /// struct.
     unsafe fn name_ptr(ptr: *mut u8) -> *mut Char16 {
         let offset_of_str = Self::name_offset();
-        ptr.add(offset_of_str) as *mut Char16
+        ptr.add(offset_of_str).cast::<Char16>()
     }
 
     /// Create a new info type in user-provided storage.
@@ -83,17 +83,18 @@ trait InfoInternal: Align + ptr::Pointee<Metadata = usize> {
 
         // Create a raw fat pointer using the `storage` as a base.
         let info_ptr: *mut Self =
-            ptr::from_raw_parts_mut(storage.as_mut_ptr() as *mut (), name_length_ucs2);
+            ptr::from_raw_parts_mut(storage.as_mut_ptr().cast::<()>(), name_length_ucs2);
 
         // Initialize the struct header.
         init(info_ptr, info_size as u64);
 
+        // Create a pointer to the part of info where the name is
+        // stored. Note that `info_ptr` is used rather than `storage` to
+        // comply with Stacked Borrows.
+        let info_name_ptr = Self::name_ptr(info_ptr.cast::<u8>());
+
         // Initialize the name slice.
-        ptr::copy(
-            name.as_ptr(),
-            Self::name_ptr(storage.as_mut_ptr()),
-            name_length_ucs2,
-        );
+        ptr::copy(name.as_ptr(), info_name_ptr, name_length_ucs2);
 
         // The struct is now valid and safe to dereference.
         let info = &mut *info_ptr;
@@ -106,10 +107,10 @@ where
     T: InfoInternal + ?Sized,
 {
     unsafe fn from_uefi<'ptr>(ptr: *mut c_void) -> &'ptr mut Self {
-        let name_ptr = Self::name_ptr(ptr as *mut u8);
+        let name_ptr = Self::name_ptr(ptr.cast::<u8>());
         let name = CStr16::from_ptr(name_ptr);
         let name_len = name.as_slice_with_nul().len();
-        &mut *ptr::from_raw_parts_mut(ptr as *mut (), name_len)
+        &mut *ptr::from_raw_parts_mut(ptr.cast::<()>(), name_len)
     }
 }
 
@@ -139,7 +140,7 @@ pub enum FileInfoCreationError {
 ///   existing file in the same directory.
 /// - If a file is read-only, the only allowed change is to remove the read-only
 ///   attribute. Other changes must be carried out in a separate transaction.
-#[derive(Debug)]
+#[derive(Debug, Eq, PartialEq)]
 #[repr(C)]
 #[unsafe_guid("09576e92-6d3f-11d2-8e39-00a0c969723b")]
 pub struct FileInfo {
@@ -219,7 +220,7 @@ impl FileInfo {
 
     /// Name of the file
     pub fn file_name(&self) -> &CStr16 {
-        unsafe { CStr16::from_ptr(&self.file_name[0]) }
+        unsafe { CStr16::from_ptr(self.file_name.as_ptr()) }
     }
 }
 
@@ -243,7 +244,7 @@ impl FileProtocolInfo for FileInfo {}
 ///
 /// Please note that only the system volume's volume label may be set using
 /// this information structure. Consider using `FileSystemVolumeLabel` instead.
-#[derive(Debug)]
+#[derive(Debug, Eq, PartialEq)]
 #[repr(C)]
 #[unsafe_guid("09576e93-6d3f-11d2-8e39-00a0c969723b")]
 pub struct FileSystemInfo {
@@ -265,7 +266,6 @@ impl FileSystemInfo {
     /// The buffer must be correctly aligned. You can query the required
     /// alignment using the `alignment()` method of the `Align` trait that this
     /// struct implements.
-    #[allow(clippy::too_many_arguments)]
     pub fn new<'buf>(
         storage: &'buf mut [u8],
         read_only: bool,
@@ -307,7 +307,7 @@ impl FileSystemInfo {
 
     /// Volume label
     pub fn volume_label(&self) -> &CStr16 {
-        unsafe { CStr16::from_ptr(&self.volume_label[0]) }
+        unsafe { CStr16::from_ptr(self.volume_label.as_ptr()) }
     }
 }
 
@@ -328,7 +328,7 @@ impl FileProtocolInfo for FileSystemInfo {}
 /// System volume label
 ///
 /// May only be obtained on the root directory's file handle.
-#[derive(Debug)]
+#[derive(Debug, Eq, PartialEq)]
 #[repr(C)]
 #[unsafe_guid("db47d7d3-fe81-11d3-9a35-0090273fc14d")]
 pub struct FileSystemVolumeLabel {
@@ -354,7 +354,7 @@ impl FileSystemVolumeLabel {
 
     /// Volume label
     pub fn volume_label(&self) -> &CStr16 {
-        unsafe { CStr16::from_ptr(&self.volume_label[0]) }
+        unsafe { CStr16::from_ptr(self.volume_label.as_ptr()) }
     }
 }
 
@@ -376,6 +376,7 @@ impl FileProtocolInfo for FileSystemVolumeLabel {}
 mod tests {
     use super::*;
     use crate::alloc_api::vec;
+    use crate::table::runtime::TimeParams;
     use crate::table::runtime::{Daylight, Time};
     use crate::CString16;
 
@@ -395,9 +396,20 @@ mod tests {
 
         let file_size = 123;
         let physical_size = 456;
-        let create_time = Time::new(1970, 1, 1, 0, 0, 0, 0, 0, Daylight::IN_DAYLIGHT);
-        let last_access_time = Time::new(1971, 1, 1, 0, 0, 0, 0, 0, Daylight::IN_DAYLIGHT);
-        let modification_time = Time::new(1972, 1, 1, 0, 0, 0, 0, 0, Daylight::IN_DAYLIGHT);
+        let tp = TimeParams {
+            year: 1970,
+            month: 1,
+            day: 1,
+            hour: 0,
+            minute: 0,
+            second: 0,
+            nanosecond: 0,
+            time_zone: None,
+            daylight: Daylight::IN_DAYLIGHT,
+        };
+        let create_time = Time::new(tp).unwrap();
+        let last_access_time = Time::new(TimeParams { year: 1971, ..tp }).unwrap();
+        let modification_time = Time::new(TimeParams { year: 1972, ..tp }).unwrap();
         let attribute = FileAttribute::READ_ONLY;
         let name = CString16::try_from("test_name").unwrap();
         let info = FileInfo::new(
