@@ -188,7 +188,7 @@ impl SystemTable<Boot> {
                 //        limitation of the NLL analysis (see Rust bug 51526).
                 let mmap_buf = &mut *(mmap_buf as *mut [u8]);
                 let mmap_comp = boot_services.memory_map(mmap_buf)?;
-                let (mmap_status, (mmap_key, mmap_iter)) = mmap_comp.split();
+                let (mmap_key, mmap_iter) = mmap_comp;
 
                 // Try to exit boot services using this memory map key
                 let result = boot_services.exit_boot_services(image, mmap_key);
@@ -199,12 +199,12 @@ impl SystemTable<Boot> {
                     continue;
                 } else {
                     // If not, report the outcome of the operation
-                    return result.map(|comp| {
+                    return result.map(|_| {
                         let st = SystemTable {
                             table: self.table,
                             _marker: PhantomData,
                         };
-                        comp.map(|_| (st, mmap_iter)).with_status(mmap_status)
+                        (st, mmap_iter)
                     });
                 }
             }
